/*
 * Dynamic Particle System Library (DPS)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _OBJECT_H_
#define _OBJECT_H_

#include "types.h"
#include "particle.h"
#include "utility.h"
#include <math.h>

/*
 * Definicao dos diferentes objectos geometricos
 * Serao definidos os seguintes: Sphere
 *
 *
 *a ordem dos tres primeiros argumentos sera no caso da accao
 * avoid: var1:look_ahead
 *          var2:magnitude
 *          var3:epsilon 
 *a ordem dos tres primeiros argumentos sera no caso da accao
 * collide: var1:one_minus_friction
 *          var2:resilience
 *          var3:cut_off_sqr
 *
 *
*/



typedef struct _DPSobject_ {
  DPSenum		type;    //esfera,rectangulo,disco,plano
  DPSenum               action;  //tipo de accao inerente ao obj:avoid,source,kill,collide
  DPSfloat              var1;    //parametros utilizados pelas accoes
  DPSfloat              var2;    //de cada objecto sendo diferentes em
  DPSfloat              var3;    //cada caso
  DPSfloat              counter; //contador de ratio
  DPSvoid              *object;   
} DPSobject; 		




//definicao do objecto esfera
typedef struct _DPSsphere_ {
  DPSvector3f		position;   //centro da esfera
  DPSfloat 		radiusIn;   //raio interior
  DPSfloat 		radiusOut;   //raio exterior
// variaveis auxiliares utilizadas posteriormente
// calculadas agora para poupar tempo
  DPSfloat              radiusInSqr;
  DPSfloat              radiusOutSqr;
} DPSsphere; 		


//definicao do objecto triangulo
typedef struct _DPStriangle_ {
  DPSvector3f           position1; //ponto1
  DPSvector3f           position2; //vector normal ao plano
  DPSvector3f           tp1;       // ponto2
  DPSvector3f           tp2;       // ponto3
  DPSvector3f 		u;         //u,v sao direccoes que definem
  DPSvector3f 		v;         //o plano do triangulo
  DPSfloat              distance;  //parametro D na equacao do plano do
                                   //triangulo(Ax+ By +Cz +D)
} DPStriangle; 		



//definicao do objecto rectangulo
typedef struct _DPSrectangle_ {
  DPSvector3f		position1;  //ponto do rectangulo
  DPSvector3f		position2;  //normal do plano
  DPSvector3f 		u;         //u,v sao direccoes que definem
  DPSvector3f 		v;         //o plano do rectangulo
  DPSfloat              distance;  //parametro D na equacao do plano do
                                   //rectangulo(Ax+ By +Cz +D)
} DPSrectangle; 		



//definicao do objecto plano
typedef struct _DPSplane_ {
  DPSvector3f		position1;   //ponto do plano
  DPSvector3f 		position2;   //vector normal ao plano  
  DPSfloat              distance;    //parametro D na equacao do plano do
                                     //rectangulo(Ax+ By +Cz +D)
   
} DPSplane; 		


//definicao do objecto cubo
typedef struct _DPScube_ {
  DPSvector3f		position1;   //pos1 canto inferior
  DPSvector3f 		position2;   //pos2 canto superior 
   
} DPScube; 		


//definicao do objecto disco
typedef struct _DPSdisc_ {
  DPSvector3f		position1;      //centro do disco
  DPSvector3f		position2;     //normal do disco
  DPSvector3f		basis;         //para calc. auxiliares
  DPSfloat 		radiusIn;       //raio interior
  DPSfloat 		radiusOut;      //raio exterior
  DPSfloat 		radiusInSqr;    //raio interior quadrado
  DPSfloat 		radiusOutSqr;   //raio exterior quadrado
  DPSvector3f 		u;             //u,v sao direccoes que definem
  DPSvector3f 		v;            //o plano do disco
  DPSfloat              distance;    //parametro D na equacao do plano do
                                    //rectangulo(Ax+ By +Cz +D)
} DPSdisc; 		

//definicao do obecto cone
typedef struct _DPScone_ {
  DPSvector3f		position1;      //centro da base
  DPSvector3f		position2;     //direccao da base ao "bico"
  DPSvector3f		basis;         //para calc. auxiliares
  DPSvector3f		tmp;          //bico do cone
  DPSfloat 		radiusIn;       //raio interior
  DPSfloat 		radiusOut;      //raio exterior
  DPSfloat 		radiusInSqr;    //raio interior quadrado
  DPSfloat 		radiusOutSqr;   //raio exterior quadrado
  DPSvector3f 		u;             //u,v sao direccoes que definem
  DPSvector3f 		v;            //o plano do disco
  DPSvector3f		n;      
} DPScone; 		


//definicao do obecto cilindro
typedef struct _DPScylinder_ {
  DPSvector3f		position1;      //centro da base
  DPSvector3f		position2;     //direccao de um centro ao outro
  DPSvector3f		basis;         //para calc. auxiliares
  DPSvector3f		tmp;            //centro da base oposta
  DPSfloat 		radiusIn;       //raio interior
  DPSfloat 		radiusOut;      //raio exterior
  DPSfloat 		radiusInSqr;    //raio interior quadrado
  DPSfloat 		radiusOutSqr;   //raio exterior quadrado
  DPSvector3f 		u;             //u,v sao direccoes que definem
  DPSvector3f 		v;            //o plano do disco
  DPSvector3f		n;      
} DPScylinder; 		









//declaracao das funcoes de inicializacao dos objectos

// os tres primeiros argumentos correspondem ao centro da esfera
// os dois seguintes sao:raio ext e raio interior
// os restantes definem a accao e os seus argumentos
//var1:look_ahead,  var2:magnitude  var3:epsilon 
DPSobject* dpsNewSphere9f(DPSfloat x, DPSfloat y, DPSfloat z, 
			  DPSfloat ri, DPSfloat ro, 
  		          DPSenum action, 
			  DPSfloat var1, DPSfloat var2, DPSfloat var3);

//um triangulo e definido por tres  pontos
// os restantes argumentos definem a accao e os seus valores
DPSobject* dpsNewTriangle13f(DPSfloat x0, DPSfloat y0, DPSfloat z0,
                          DPSfloat x1, DPSfloat y1, DPSfloat z1, 
			  DPSfloat x2, DPSfloat y2, DPSfloat z2,
  		          DPSenum action, 
			  DPSfloat var1, DPSfloat var2, DPSfloat var3);

//um rectangulo e definido por um ponto e dois vectores directores
DPSobject* dpsNewRectangle13f(DPSfloat x0, DPSfloat y0, DPSfloat z0,
                          DPSfloat x1, DPSfloat y1, DPSfloat z1, 
			  DPSfloat x2, DPSfloat y2, DPSfloat z2,
  		          DPSenum action, 
			  DPSfloat var1, DPSfloat var2, DPSfloat var3);

//um plano e definido por um ponto e um vector director
DPSobject* dpsNewPlane10f(DPSfloat x0, DPSfloat y0, DPSfloat z0,
                        DPSfloat x1, DPSfloat y1, DPSfloat z1, 
		         DPSenum action, 
			  DPSfloat var1, DPSfloat var2, DPSfloat var3);


//umc cubo e definido por dois pontos
DPSobject* dpsNewCube10f(DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                        DPSfloat x1, DPSfloat y1, DPSfloat z1, 
  		        DPSenum action, 
			DPSfloat var1, DPSfloat var2, DPSfloat var3);

//um disco e'por um ponto, um vector normal e
// um raio interior e um raio exterior 
DPSobject* dpsNewDisc12f(DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                        DPSfloat x1, DPSfloat y1, DPSfloat z1, 
			DPSfloat ri, DPSfloat ro,
  		        DPSenum action, 
			DPSfloat var1, DPSfloat var2, DPSfloat var3);


//um cone e' definido por um ponto que define o centro da base
//um por um ponto que corresponde ao "bico" do cone
//por um raio interior e um raio exterior
DPSobject* dpsNewCone12f(DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                         DPSfloat x1, DPSfloat y1, DPSfloat z1, 
			 DPSfloat ri, DPSfloat ro,
  		         DPSenum action, 
			 DPSfloat var1, DPSfloat var2, DPSfloat var3);


//um cone e' definido por um ponto que define o centro da base
//um por um ponto que corresponde ao centro da outra extremidade
//por um raio interior e um raio exterior
DPSobject* dpsNewCylinder12f(DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                             DPSfloat x1, DPSfloat y1, DPSfloat z1, 
		      	     DPSfloat ri, DPSfloat ro,
  		             DPSenum action, 
			     DPSfloat var1, DPSfloat var2, DPSfloat var3);





//funcao que verifica se um dado ponto se encontra dentro de uma esfera
DPSboolean isWithinSphere(DPSfloat x, DPSfloat y, DPSfloat z, DPSsphere *sph);


//funcao a ser utilizada no exemplo
void dpsApplyObject(DPSobject *object, GList** particles, DPSstate *state, DPSsets sets);


#endif

